@@ -103,11 +103,11 @@ maybeToM err Nothing = Fail.fail err
 maybeToM _ (Just x) = return x
 
 -- | Convert a 'Maybe' value to a value in any monad
-eitherToM :: Monad m
+eitherToM :: MonadFail m
           => String -- ^ Error message to fail with for 'Nothing'
           -> Either String a -- ^ The 'Maybe' value.
           -> m a -- ^ The resulting value in the monad.
-eitherToM err (Left s)  = fail (err ++ ": " ++ s)
+eitherToM err (Left s)  = Fail.fail (err ++ ": " ++ s)
 eitherToM   _ (Right x) = return x
 
 -- | The format for \"dateTime.iso8601\"
@@ -136,14 +136,14 @@ handleError h m = do
                   Right x <- runExceptT (catchError m (lift . h))
                   return x
 
-errorRead :: (Monad m, Read a) =>
+errorRead :: (MonadFail m, Read a) =>
              ReadS a -- ^ Parser
           -> String -- ^ Error message
           -> String -- ^ String to parse
           -> Err m a
 errorRead r err s = case [x | (x,t) <- r s, ("","") <- lex t] of
                           [x] -> return x
-                          _   -> fail (err ++ ": '" ++ s ++ "'")
+                          _   -> Fail.fail (err ++ ": '" ++ s ++ "'")
 
 --
 -- Types for methods calls and responses
@@ -170,6 +170,7 @@ data Value =
     | ValueBase64 BS.ByteString -- ^ base 64.  NOTE that you should provide the raw data; the haxr library takes care of doing the base-64 encoding.
     | ValueStruct [(String,Value)] -- ^ struct
     | ValueArray [Value]  -- ^ array
+    | ValueNil -- ^ nil
       deriving (Eq, Show) -- for debugging
 
 -- | An XML-RPC value. Use for error messages and introspection.
@@ -183,6 +184,7 @@ data Type =
           | TStruct
           | TArray
           | TUnknown
+          | TNil
       deriving (Eq)
 
 instance Show Type where
@@ -195,6 +197,7 @@ instance Show Type where
     show TStruct = "struct"
     show TArray = "array"
     show TUnknown = "unknown"
+    show TNil = "nil"
 
 instance Read Type where
     readsPrec _ s = case break isSpace (dropWhile isSpace s) of
@@ -206,6 +209,7 @@ instance Read Type where
                     ("base64",r) -> [(TBase64,r)]
                     ("struct",r) -> [(TStruct,r)]
                     ("array",r) -> [(TArray,r)]
+                    ("nil",r) -> [(TNil,r)]
 
 -- | Gets the value of a struct member
 structGetValue :: MonadFail m => String -> Value -> Err m Value
@@ -222,10 +226,10 @@ faultStruct code str = ValueStruct [("faultCode",ValueInt code),
 -- "The body of the response is a single XML structure, a
 -- <methodResponse>, which can contain a single <params> which contains a
 -- single <param> which contains a single <value>."
-onlyOneResult :: Monad m => [Value] -> Err m Value
-onlyOneResult [] = fail "Method returned no result"
+onlyOneResult :: MonadFail m => [Value] -> Err m Value
+onlyOneResult [] = Fail.fail "Method returned no result"
 onlyOneResult [x] = return x
-onlyOneResult _ = fail "Method returned more than one result"
+onlyOneResult _ = Fail.fail "Method returned more than one result"
 
 --
 -- Converting to and from XML-RPC types
@@ -237,12 +241,12 @@ class XmlRpcType a where
     toValue :: a -> Value
     -- | Convert from a 'Value' to this type. May fail if
     --   if there is a type error.
-    fromValue :: Monad m => Value -> Err m a
+    fromValue :: MonadFail m => Value -> Err m a
     getType :: a -> Type
 
-typeError :: (XmlRpcType a, Monad m) => Value -> Err m a
+typeError :: (XmlRpcType a, MonadFail m) => Value -> Err m a
 typeError v = withType $ \t ->
-       fail ("Wanted: "
+       Fail.fail ("Wanted: "
              ++ show (getType t)
              ++ "', got: '"
              ++ showXml False (toXRValue v) ++ "'") `asTypeOf` return t
@@ -251,7 +255,7 @@ typeError v = withType $ \t ->
 withType :: (a -> Err m a) -> Err m a
 withType f = f undefined
 
-simpleFromValue :: (Monad m, XmlRpcType a) => (Value -> Maybe a)
+simpleFromValue :: (MonadFail m, XmlRpcType a) => (Value -> Maybe a)
                 -> Value -> Err m a
 simpleFromValue f v =
     maybe (typeError v) return (f v)
@@ -319,6 +323,13 @@ instance XmlRpcType CalendarTime where
     fromValue = liftM localTimeToCalendarTime . fromValue
     getType _ = TDateTime
 
+instance XmlRpcType () where
+    toValue = const ValueNil
+    fromValue = simpleFromValue f
+        where f ValueNil = Just ()
+              f _ = Nothing
+    getType _ = TNil
+
 -- FIXME: array elements may have different types
 instance OVERLAPPABLE_ XmlRpcType a => XmlRpcType [a] where
     toValue = ValueArray . map toValue
@@ -378,7 +389,7 @@ getField x xs = maybeToM ("struct member " ++ show x ++ " not found")
                 (lookup x xs) >>= fromValue
 
 -- | Get a field value from a (possibly heterogeneous) struct.
-getFieldMaybe :: (Monad m, XmlRpcType a) =>
+getFieldMaybe :: (MonadFail m, XmlRpcType a) =>
             String           -- ^ Field name
          -> [(String,Value)] -- ^ Struct
          -> Err m (Maybe a)
@@ -402,6 +413,7 @@ toXRValue (ValueBase64 s) = XR.Value [XR.Value_Base64 (XR.Base64 (showBase64 s))
 toXRValue (ValueStruct xs) = XR.Value [XR.Value_Struct (XR.Struct (map toXRMember xs))]
 toXRValue (ValueArray xs) =
     XR.Value [XR.Value_Array (XR.Array (XR.Data (map toXRValue xs)))]
+toXRValue ValueNil = XR.Value [XR.Value_Nil (XR.Nil ())]
 
 showInt :: Int -> String
 showInt = show
@@ -443,7 +455,7 @@ toXRMember (n, v) = XR.Member (XR.Name n) (toXRValue v)
 -- Converting from XR types
 --
 
-fromXRValue :: Monad m => XR.Value -> Err m Value
+fromXRValue :: MonadFail m => XR.Value -> Err m Value
 fromXRValue (XR.Value vs)
   =  case (filter notstr vs) of
        []     -> liftM  (ValueUnwrapped . concat) (mapM (readString . unstr) vs)
@@ -467,9 +479,10 @@ fromXRValue (XR.Value vs)
     liftM ValueStruct (mapM fromXRMember ms)
   f (XR.Value_Array (XR.Array (XR.Data xs))) =
     liftM ValueArray (mapM fromXRValue xs)
+  f (XR.Value_Nil (XR.Nil x)) = return ValueNil
 
 
-fromXRMember :: Monad m => XR.Member -> Err m (String,Value)
+fromXRMember :: MonadFail m => XR.Member -> Err m (String,Value)
 fromXRMember (XR.Member (XR.Name n) xv) = liftM (\v -> (n,v)) (fromXRValue xv)
 
 -- | From the XML-RPC specification:
@@ -478,14 +491,14 @@ fromXRMember (XR.Member (XR.Name n) xv) = liftM (\v -> (n,v)) (fromXRValue xv)
 -- minus at the beginning of a string of numeric characters. Leading
 -- zeros are collapsed. Whitespace is not permitted. Just numeric
 -- characters preceeded by a plus or minus.\"
-readInt :: Monad m => String -> Err m Int
+readInt :: MonadFail m => String -> Err m Int
 readInt s = errorRead reads "Error parsing integer" s
 
 
 -- | From the XML-RPC specification:
 --
 -- \"0 (false) or 1 (true)\"
-readBool :: Monad m => String -> Err m Bool
+readBool :: MonadFail m => String -> Err m Bool
 readBool s = errorRead readsBool "Error parsing boolean" s
     where readsBool "true" = [(True,"")]
           readsBool "false" = [(False,"")]
@@ -516,7 +529,7 @@ readString = return . replace "&amp;" "&" . replace "&lt;" "<"
 -- is implementation-dependent, is not specified.
 --
 -- FIXME: accepts more than decimal point notation
-readDouble :: Monad m => String -> Err m Double
+readDouble :: MonadFail m => String -> Err m Double
 readDouble s = errorRead reads "Error parsing double" s
 
 -- | From <http://groups.yahoo.com/group/xml-rpc/message/4733>:
@@ -525,10 +538,10 @@ readDouble s = errorRead reads "Error parsing double" s
 --   content of this element should not be assumed to comply with the
 --   variants of the ISO8601 standard. Only assume YYYYMMDDTHH:mm:SS\"
 -- FIXME: make more robust
-readDateTime :: Monad m => String -> Err m LocalTime
+readDateTime :: MonadFail m => String -> Err m LocalTime
 readDateTime dt =
     maybe
-        (fail $ "Error parsing dateTime '" ++ dt ++ "'")
+        (Fail.fail $ "Error parsing dateTime '" ++ dt ++ "'")
         return
         (parseTimeM True defaultTimeLocale xmlRpcDateFormat dt)
 
@@ -566,10 +579,10 @@ calendarTimeToLocalTime ct =
 readBase64 :: Monad m => String -> Err m BS.ByteString
 readBase64 = return . Base64.decode . BS.pack
 
-fromXRParams :: Monad m => XR.Params -> Err m [Value]
+fromXRParams :: MonadFail m => XR.Params -> Err m [Value]
 fromXRParams (XR.Params xps) = mapM (\(XR.Param v) -> fromXRValue v) xps
 
-fromXRMethodCall :: Monad m => XR.MethodCall -> Err m MethodCall
+fromXRMethodCall :: MonadFail m => XR.MethodCall -> Err m MethodCall
 fromXRMethodCall (XR.MethodCall (XR.MethodName name) params) =
     liftM (MethodCall name) (fromXRParams (fromMaybe (XR.Params []) params))
 
@@ -590,7 +603,7 @@ fromXRMethodResponse (XR.MethodResponseFault (XR.Fault v)) =
 --
 
 -- | Parses a method call from XML.
-parseCall :: (Show e, MonadError e m) => String -> Err m MethodCall
+parseCall :: (Show e, MonadError e m, MonadFail m) => String -> Err m MethodCall
 parseCall c =
     do
     mxc <- errorToErr (readXml c)
